@@ -1,7 +1,8 @@
 from .components import ActionRow, Button, SelectMenu, SelectMenuOption
 from .user import User
 from .message import Message
-
+from .. import utils
+from ..embed import Embed
 
 class Context:
     def __init__(self, socket, data):
@@ -18,14 +19,15 @@ class Context:
         self.id = self.raw.get('id', None)
         self.token = self.raw.get('token', None)
         self.type = self.raw.get('type', None)
+        self.invoker = User(self.raw['member']['user'])
         self.callback_url = f"https://discord.com/api/v8/interactions/{self.id}/{self.token}/callback"
 
-    async def callback(self, content: str = '', embeds: list = [], components: list = [], mentions: list = [], ephemeral: bool= False):
+    async def callback(self, content: str = '', embeds: list = [], components: list = [], mentions: list = [], ephemeral: bool= False, type: int = utils.CHANNEL_WITH_SOURCE):
         message = {
-            "type": 4,
+            "type": type,
             "data": {
                 "content": content,
-                "embeds": embeds,
+                "embeds": [e.build() for e in embeds if isinstance(e, Embed)],
                 "allowed_mentions": mentions,
             }
         }
@@ -37,7 +39,7 @@ class Context:
             for action_row in components:
                 if isinstance(action_row, ActionRow):
                     built_action_rows.append(action_row.build())
-        
+
             fully_built = []
             for bar in built_action_rows:
                 new_ar = {"type": 1, "components": []}
@@ -64,15 +66,15 @@ class SelectMenuContext:
         self.ucid = self.raw['data']['custom_id']
         self.values = self.raw['data']['values']
         self.used = User(self.raw['member']['user'])
-        self.invoked = User(data['message']['interaction']['user'])
+        self.invoker = User(data['message']['interaction']['user'])
         self.callback_url = f"https://discord.com/api/v8/interactions/{self.raw['id']}/{self.raw['token']}/callback"
 
-    async def callback(self, content: str = '', embeds: list = [], components: list = [], mentions: list = [], ephemeral: bool= False):
+    async def callback(self, content: str = '', embeds: list = [], components: list = [], mentions: list = [], ephemeral: bool= False, type: int = utils.CHANNEL_WITH_SOURCE):
         message = {
-            "type": 4,
+            "type": type,
             "data": {
                 "content": content,
-                "embeds": embeds,
+                "embeds": [e.build() for e in embeds if isinstance(e, Embed)],
                 "allowed_mentions": mentions,
             }
         }
@@ -84,7 +86,7 @@ class SelectMenuContext:
             for action_row in components:
                 if isinstance(action_row, ActionRow):
                     built_action_rows.append(action_row.build())
-        
+
             fully_built = []
             for bar in built_action_rows:
                 new_ar = {"type": 1, "components": []}
@@ -114,16 +116,16 @@ class ButtonContext:
         self.socket = socket
         self.raw = data
         self.ucid = self.raw['data']['custom_id']
-        self.invoked = User(data['message']['interaction']['user'])
+        self.invoker = User(data['message']['interaction']['user'])
         self.used = User(data['member']['user'])
         self.callback_url = f"https://discord.com/api/v8/interactions/{self.raw['id']}/{self.raw['token']}/callback"
 
-    async def callback(self, content: str = '', embeds: list = [], components: list = [], mentions: list = [], ephemeral: bool= False):
+    async def callback(self, content: str = '', embeds: list = [], components: list = [], mentions: list = [], ephemeral: bool= False, type: int = utils.CHANNEL_WITH_SOURCE):
         message = {
-            "type": 4,
+            "type": type,
             "data": {
                 "content": content,
-                "embeds": embeds,
+                "embeds": [e.build() for e in embeds if isinstance(e, Embed)],
                 "allowed_mentions": mentions,
             }
         }
@@ -135,7 +137,7 @@ class ButtonContext:
             for action_row in components:
                 if isinstance(action_row, ActionRow):
                     built_action_rows.append(action_row.build())
-        
+
             fully_built = []
             for bar in built_action_rows:
                 new_ar = {"type": 1, "components": []}
